@@ -24,7 +24,6 @@
 
 #include "pango-glyph-item.h"
 #include "pango-impl-utils.h"
-#include "pango-attributes-private.h"
 
 #define LTR(glyph_item) (((glyph_item)->item->analysis.level % 2) == 0)
 
@@ -132,11 +131,11 @@ pango_glyph_item_split (PangoGlyphItem *orig,
 
 /**
  * pango_glyph_item_copy:
- * @orig: (nullable): a #PangoGlyphItem, may be %NULL
+ * @orig: a #PangoGlyphItem, may be %NULL
  *
  * Make a deep copy of an existing #PangoGlyphItem structure.
  *
- * Return value: (nullable): the newly allocated #PangoGlyphItem, which should
+ * Return value: the newly allocated #PangoGlyphItem, which should
  *               be freed with pango_glyph_item_free(), or %NULL
  *               if @orig was %NULL.
  *
@@ -160,7 +159,7 @@ pango_glyph_item_copy  (PangoGlyphItem *orig)
 
 /**
  * pango_glyph_item_free:
- * @glyph_item: (nullable): a #PangoGlyphItem, may be %NULL
+ * @glyph_item: a #PangoGlyphItem, may be %NULL
  *
  * Frees a #PangoGlyphItem and resources to which it points.
  *
@@ -187,11 +186,11 @@ G_DEFINE_BOXED_TYPE (PangoGlyphItem, pango_glyph_item,
 
 /**
  * pango_glyph_item_iter_copy:
- * @orig: (nullable): a #PangoGlyphItemIter, may be %NULL
+ * @orig: a #PangoGlyphItemIter, may be %NULL
  *
  * Make a shallow copy of an existing #PangoGlyphItemIter structure.
  *
- * Return value: (nullable): the newly allocated #PangoGlyphItemIter, which should
+ * Return value: the newly allocated #PangoGlyphItemIter, which should
  *               be freed with pango_glyph_item_iter_free(), or %NULL
  *               if @orig was %NULL.
  *
@@ -214,7 +213,7 @@ pango_glyph_item_iter_copy  (PangoGlyphItemIter *orig)
 
 /**
  * pango_glyph_item_iter_free:
- * @iter: (nullable): a #PangoGlyphItemIter, may be %NULL
+ * @iter: a #PangoGlyphItemIter, may be %NULL
  *
  * Frees a #PangoGlyphItemIter created by pango_glyph_item_iter_copy().
  *
@@ -282,7 +281,7 @@ pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 
-	  if (glyphs->log_clusters[glyph_index] > cluster)
+	  if (glyphs->log_clusters[glyph_index] != cluster)
 	    {
 	      iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->end_char += pango_utf8_strlen (iter->text + iter->start_index,
@@ -305,7 +304,7 @@ pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 
-	  if (glyphs->log_clusters[glyph_index] > cluster)
+	  if (glyphs->log_clusters[glyph_index] != cluster)
 	    {
 	      iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->end_char += pango_utf8_strlen (iter->text + iter->start_index,
@@ -316,10 +315,6 @@ pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
     }
 
   iter->end_glyph = glyph_index;
-
-  g_assert (iter->start_char <= iter->end_char);
-  g_assert (iter->end_char <= item->num_chars);
-
   return TRUE;
 }
 
@@ -364,6 +359,8 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
       cluster = glyphs->log_clusters[glyph_index - 1];
       while (TRUE)
 	{
+	  glyph_index--;
+
 	  if (glyph_index == 0)
 	    {
 	      iter->start_index = item->offset;
@@ -371,9 +368,7 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 
-	  glyph_index--;
-
-	  if (glyphs->log_clusters[glyph_index] < cluster)
+	  if (glyphs->log_clusters[glyph_index] != cluster)
 	    {
 	      glyph_index++;
 	      iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
@@ -388,6 +383,8 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
       cluster = glyphs->log_clusters[glyph_index + 1];
       while (TRUE)
 	{
+	  glyph_index++;
+
 	  if (glyph_index == glyphs->num_glyphs - 1)
 	    {
 	      iter->start_index = item->offset;
@@ -395,9 +392,7 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 
-	  glyph_index++;
-
-	  if (glyphs->log_clusters[glyph_index] < cluster)
+	  if (glyphs->log_clusters[glyph_index] != cluster)
 	    {
 	      glyph_index--;
 	      iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
@@ -409,10 +404,6 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
     }
 
   iter->start_glyph = glyph_index;
-
-  g_assert (iter->start_char <= iter->end_char);
-  g_assert (0 <= iter->start_char);
-
   return TRUE;
 }
 
@@ -585,13 +576,12 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 			      const char       *text,
 			      PangoAttrList    *list)
 {
-  PangoAttrIterator iter;
+  PangoAttrIterator *iter = pango_attr_list_get_iterator (list);
   GSList *result = NULL;
   ApplyAttrsState state;
   gboolean start_new_segment = FALSE;
   gboolean have_cluster;
   int range_start, range_end;
-  gboolean is_ellipsis;
 
   /* This routine works by iterating through the item cluster by
    * cluster; we accumulate the attributes that we need to
@@ -608,25 +598,21 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 
   /* Advance the attr iterator to the start of the item
    */
-  _pango_attr_list_get_iterator (list, &iter);
   do
     {
-      pango_attr_iterator_range (&iter, &range_start, &range_end);
+      pango_attr_iterator_range (iter, &range_start, &range_end);
       if (range_end > glyph_item->item->offset)
 	break;
     }
-  while (pango_attr_iterator_next (&iter));
+  while (pango_attr_iterator_next (iter));
 
-  state.segment_attrs = pango_attr_iterator_get_attrs (&iter);
-
-  is_ellipsis = (glyph_item->item->analysis.flags & PANGO_ANALYSIS_FLAG_IS_ELLIPSIS) != 0;
+  state.segment_attrs = pango_attr_iterator_get_attrs (iter);
 
   /* Short circuit the case when we don't actually need to
    * split the item
    */
-  if (is_ellipsis ||
-      (range_start <= glyph_item->item->offset &&
-       range_end >= glyph_item->item->offset + glyph_item->item->length))
+  if (range_start <= glyph_item->item->offset &&
+      range_end >= glyph_item->item->offset + glyph_item->item->length)
     goto out;
 
   for (have_cluster = pango_glyph_item_iter_init_start (&state.iter, glyph_item, text);
@@ -646,7 +632,7 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 	{
 	  result = g_slist_prepend (result,
 				    split_before_cluster_start (&state));
-	  state.segment_attrs = pango_attr_iterator_get_attrs (&iter);
+	  state.segment_attrs = pango_attr_iterator_get_attrs (iter);
 	}
 
       start_new_segment = FALSE;
@@ -665,8 +651,8 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 	   */
 	  start_new_segment = TRUE;
 
-	  have_next = pango_attr_iterator_next (&iter);
-	  pango_attr_iterator_range (&iter, &range_start, &range_end);
+	  have_next = pango_attr_iterator_next (iter);
+	  pango_attr_iterator_range (iter, &range_start, &range_end);
 
 	  if (range_start >= state.iter.end_index) /* New range doesn't intersect this cluster */
 	    {
@@ -690,7 +676,7 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 	    }
 
 	  state.segment_attrs = g_slist_concat (state.segment_attrs,
-						pango_attr_iterator_get_attrs (&iter));
+						pango_attr_iterator_get_attrs (iter));
 	}
       while (have_next);
     }
@@ -704,7 +690,7 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
   if (LTR (glyph_item))
     result = g_slist_reverse (result);
 
-  _pango_attr_iterator_destroy (&iter);
+  pango_attr_iterator_destroy (iter);
 
   return result;
 }
@@ -715,8 +701,8 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
  * @text: text that @glyph_item corresponds to
  *   (glyph_item->item->offset is an offset from the
  *    start of @text)
- * @log_attrs: (array): logical attributes for the item
- *   (the first logical attribute refers to the position
+ * @log_attrs: logical attributes for the item (the
+ *   first logical attribute refers to the position
  *   before the first character in the item)
  * @letter_spacing: amount of letter spacing to add
  *   in Pango units. May be negative, though too large
@@ -788,10 +774,9 @@ pango_glyph_item_letter_space (PangoGlyphItem *glyph_item,
  * @text: text that @glyph_item corresponds to
  *   (glyph_item->item->offset is an offset from the
  *    start of @text)
- * @logical_widths: (array): an array whose length is the number of
- *                  characters in glyph_item (equal to
- *                  glyph_item->item->num_chars) to be filled in with
- *                  the resulting character widths.
+ * @logical_widths: an array whose length is the number of characters in
+ *                  glyph_item (equal to glyph_item->item->num_chars)
+ *                  to be filled in with the resulting character widths.
  *
  * Given a #PangoGlyphItem and the corresponding
  * text, determine the screen width corresponding to each character. When
