@@ -19,147 +19,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:pangocairo
- * @short_description:Font handling and rendering with Cairo
- * @title:Cairo Fonts and Rendering
- *
- * The <ulink url="http://cairographics.org">Cairo library</ulink> is a
- * vector graphics library with a powerful rendering model. It has such
- * features as anti-aliased primitives, alpha-compositing, and
- * gradients. Multiple backends for Cairo are available, to allow
- * rendering to images, to PDF files, and to the screen on X and on other
- * windowing systems. The functions in this section allow using Pango
- * to render to Cairo surfaces.
- *
- * Using Pango with Cairo is straightforward. A #PangoContext created
- * with pango_cairo_font_map_create_context() can be used on any
- * Cairo context (cairo_t), but needs to be updated to match the
- * current transformation matrix and target surface of the Cairo context
- * using pango_cairo_update_context(). The convenience functions
- * pango_cairo_create_layout() and pango_cairo_update_layout() handle
- * the common case where the program doesn't need to manipulate the
- * properties of the #PangoContext.
- *
- * When you get the metrics of a layout or of a piece of a layout using
- * functions such as pango_layout_get_extents(), the reported metrics
- * are in user-space coordinates. If a piece of text is 10 units long,
- * and you call cairo_scale (cr, 2.0), it still is more-or-less 10
- * units long. However, the results will be affected by hinting
- * (that is, the process of adjusting the text to look good on the
- * pixel grid), so you shouldn't assume they are completely independent
- * of the current transformation matrix. Note that the basic metrics
- * functions in Pango report results in integer Pango units. To get
- * to the floating point units used in Cairo divide by %PANGO_SCALE.
- *
- * ## Using Pango with Cairo ## {#rotated-example}
- *
- * |[<!-- language="C" -->
- * #include <math.h>
- * #include <pango/pangocairo.h>
- *
- * static void
- * draw_text (cairo_t *cr)
- * {
- * #define RADIUS 150
- * #define N_WORDS 10
- * #define FONT "Sans Bold 27"
- *
- *   PangoLayout *layout;
- *   PangoFontDescription *desc;
- *   int i;
- *
- *   /&ast; Center coordinates on the middle of the region we are drawing
- *    &ast;/
- *   cairo_translate (cr, RADIUS, RADIUS);
- *
- *   /&ast; Create a PangoLayout, set the font and text &ast;/
- *   layout = pango_cairo_create_layout (cr);
- *
- *   pango_layout_set_text (layout, "Text", -1);
- *   desc = pango_font_description_from_string (FONT);
- *   pango_layout_set_font_description (layout, desc);
- *   pango_font_description_free (desc);
- *
- *   /&ast; Draw the layout N_WORDS times in a circle &ast;/
- *   for (i = 0; i &lt; N_WORDS; i++)
- *     {
- *       int width, height;
- *       double angle = (360. * i) / N_WORDS;
- *       double red;
- *
- *       cairo_save (cr);
- *
- *       /&ast; Gradient from red at angle == 60 to blue at angle == 240 &ast;/
- *       red   = (1 + cos ((angle - 60) * G_PI / 180.)) / 2;
- *       cairo_set_source_rgb (cr, red, 0, 1.0 - red);
- *
- *       cairo_rotate (cr, angle * G_PI / 180.);
- *
- *       /&ast; Inform Pango to re-layout the text with the new transformation &ast;/
- *       pango_cairo_update_layout (cr, layout);
- *
- *       pango_layout_get_size (layout, &amp;width, &amp;height);
- *       cairo_move_to (cr, - ((double)width / PANGO_SCALE) / 2, - RADIUS);
- *       pango_cairo_show_layout (cr, layout);
- *
- *       cairo_restore (cr);
- *     }
- *
- *   /&ast; free the layout object &ast;/
- *   g_object_unref (layout);
- * }
- *
- * int main (int argc, char **argv)
- * {
- *   cairo_t *cr;
- *   char *filename;
- *   cairo_status_t status;
- *   cairo_surface_t *surface;
- *
- *   if (argc != 2)
- *     {
- *       g_printerr ("Usage: cairosimple OUTPUT_FILENAME\n");
- *       return 1;
- *     }
- *
- *      filename = argv[1];
- *
- *   surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
- *                                         2 * RADIUS, 2 * RADIUS);
- *   cr = cairo_create (surface);
- *
- *   cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
- *   cairo_paint (cr);
- *   draw_text (cr);
- *   cairo_destroy (cr);
- *
- *   status = cairo_surface_write_to_png (surface, filename);
- *   cairo_surface_destroy (surface);
- *
- *   if (status != CAIRO_STATUS_SUCCESS)
- *     {
- *       g_printerr ("Could not save png to '%s'\n", filename);
- *       return 1;
- *     }
- *
- *   return 0;
- * }
- * ]|
- *
- * Once you build and run the example code above, you should see the
- * following result:
- *
- * ![Output of rotated-example](rotated-text.png)
- */
 #include "config.h"
 
 #include <math.h>
 
-#include "pango-font-private.h"
 #include "pangocairo-private.h"
 #include "pango-glyph-item.h"
-#include "pango-impl-utils.h"
 
 typedef struct _PangoCairoRendererClass PangoCairoRendererClass;
 
@@ -193,38 +58,12 @@ set_color (PangoCairoRenderer *crenderer,
 	   PangoRenderPart     part)
 {
   PangoColor *color = pango_renderer_get_color ((PangoRenderer *) (crenderer), part);
-  guint16 a = pango_renderer_get_alpha ((PangoRenderer *) (crenderer), part);
-  gdouble red, green, blue, alpha;
-
-  if (!a && !color)
-    return;
 
   if (color)
-    {
-      red = color->red / 65535.;
-      green = color->green / 65535.;
-      blue = color->blue / 65535.;
-      alpha = 1.;
-    }
-  else
-    {
-      cairo_pattern_t *pattern = cairo_get_source (crenderer->cr);
-
-      if (pattern && cairo_pattern_get_type (pattern) == CAIRO_PATTERN_TYPE_SOLID)
-        cairo_pattern_get_rgba (pattern, &red, &green, &blue, &alpha);
-      else
-        {
-          red = 0.;
-          green = 0.;
-          blue = 0.;
-          alpha = 1.;
-        }
-    }
-
-  if (a)
-    alpha = a / 65535.;
-
-  cairo_set_source_rgba (crenderer->cr, red, green, blue, alpha);
+    cairo_set_source_rgb (crenderer->cr,
+			  color->red / 65535.,
+			  color->green / 65535.,
+			  color->blue / 65535.);
 }
 
 /* note: modifies crenderer->cr without doing cairo_save/restore() */
@@ -289,7 +128,7 @@ _pango_cairo_renderer_draw_frame (PangoCairoRenderer *crenderer,
 	  alpha = G_PI_2 - alpha;
 	  tan_alpha2 = tan (alpha * .5);
 	  if (tan_alpha2 < 1e-5 || (sy = d2 / tan_alpha2, 2. * sy > height - d))
-	    sy = (height - d) * .5;
+	    sy = (width - d) * .5;
 
 	  cos_alpha = cos (alpha);
 	  if (cos_alpha < 1e-5 || (sx = d2 / cos_alpha, 2. * sx > width - d))
@@ -369,13 +208,10 @@ _pango_cairo_renderer_draw_unknown_glyph (PangoCairoRenderer *crenderer,
   double x0, y0;
   int row, col;
   int rows, cols;
-  double width, lsb;
-  char hexbox_string[2] = { 0, 0 };
+  char hexbox_string[2] = {0, 0};
   PangoCairoFontHexBoxInfo *hbi;
   gunichar ch;
   gboolean invalid_input;
-  const char *p;
-  const char *name;
 
   cairo_save (crenderer->cr);
 
@@ -389,110 +225,21 @@ _pango_cairo_renderer_draw_unknown_glyph (PangoCairoRenderer *crenderer,
       goto done;
     }
 
+  rows = hbi->rows;
   if (G_UNLIKELY (invalid_input))
     {
-      rows = hbi->rows;
       cols = 1;
     }
-  else if (ch == 0x2423 ||
-           g_unichar_type (ch) == G_UNICODE_SPACE_SEPARATOR)
-    {
-      /* We never want to show a hex box or other drawing for
-       * space. If we want space to be visible, we replace 0x20
-       * by 0x2423 (visible space).
-       *
-       * Since we don't want to rely on glyph availability,
-       * we render a centered dot ourselves.
-       */
-      double x = cx + 0.5 *((double)gi->geometry.width / PANGO_SCALE);
-      double y = cy + hbi->box_descent - 0.5 * hbi->box_height;
-
-      cairo_new_sub_path (crenderer->cr);
-      cairo_arc (crenderer->cr, x, y, 1.5 * hbi->line_width, 0, 2 * G_PI);
-      cairo_close_path (crenderer->cr);
-      cairo_fill (crenderer->cr);
-      goto done;
-    }
-  else if (ch == '\t')
-    {
-      /* Since we don't want to rely on glyph availability,
-       * we render an arrow like ↦ ourselves.
-       */
-      double y = cy + hbi->box_descent - 0.5 * hbi->box_height;
-      double width = (double)gi->geometry.width / PANGO_SCALE;
-      double offset = 0.2 * width;
-      double x = cx + offset;
-      double al = width - 2 * offset; /* arrow length */
-      double tl = MIN (hbi->digit_width, 0.75 * al); /* tip length */
-      double tw2 = 2.5 * hbi->line_width; /* tip width / 2 */
-      double lw2 = 0.5 * hbi->line_width; /* line width / 2 */
-
-      cairo_move_to (crenderer->cr, x - lw2, y - tw2);
-      cairo_line_to (crenderer->cr, x + lw2, y - tw2);
-      cairo_line_to (crenderer->cr, x + lw2, y - lw2);
-      cairo_line_to (crenderer->cr, x + al - tl, y - lw2);
-      cairo_line_to (crenderer->cr, x + al - tl, y - tw2);
-      cairo_line_to (crenderer->cr, x + al,  y);
-      cairo_line_to (crenderer->cr, x + al - tl, y + tw2);
-      cairo_line_to (crenderer->cr, x + al - tl, y + lw2);
-      cairo_line_to (crenderer->cr, x + lw2, y + lw2);
-      cairo_line_to (crenderer->cr, x + lw2, y + tw2);
-      cairo_line_to (crenderer->cr, x - lw2, y + tw2);
-      cairo_close_path (crenderer->cr);
-      cairo_fill (crenderer->cr);
-      goto done;
-    }
-  else if (ch == '\n' || ch == 0x2028 || ch == 0x2029)
-    {
-      /* Since we don't want to rely on glyph availability,
-       * we render an arrow like ↵ ourselves.
-       */
-      double width = (double)gi->geometry.width / PANGO_SCALE;
-      double offset = 0.2 * width;
-      double al = width - 2 * offset; /* arrow length */
-      double tl = MIN (hbi->digit_width, 0.75 * al); /* tip length */
-      double ah = al - 0.5 * tl; /* arrow height */
-      double tw2 = 2.5 * hbi->line_width; /* tip width / 2 */
-      double x = cx + offset;
-      double y = cy - (hbi->box_height - al) / 2;
-      double lw2 = 0.5 * hbi->line_width; /* line width / 2 */
-
-      cairo_move_to (crenderer->cr, x, y);
-      cairo_line_to (crenderer->cr, x + tl, y - tw2);
-      cairo_line_to (crenderer->cr, x + tl, y - lw2);
-      cairo_line_to (crenderer->cr, x + al - lw2, y - lw2);
-      cairo_line_to (crenderer->cr, x + al - lw2, y - ah);
-      cairo_line_to (crenderer->cr, x + al + lw2, y - ah);
-      cairo_line_to (crenderer->cr, x + al + lw2, y + lw2);
-      cairo_line_to (crenderer->cr, x + tl, y + lw2);
-      cairo_line_to (crenderer->cr, x + tl, y + tw2);
-      cairo_close_path (crenderer->cr);
-      cairo_fill (crenderer->cr);
-      goto done;
-    }
-  else if ((name = pango_get_ignorable_size (ch, &rows, &cols)))
-    {
-      /* Nothing else to do, we render 'default ignorable' chars
-       * as hex box with their nick.
-       */
-    }
   else
     {
-      /* Everything else gets a traditional hex box. */
-      rows = hbi->rows;
       cols = (ch > 0xffff ? 6 : 4) / rows;
       g_snprintf (buf, sizeof(buf), (ch > 0xffff) ? "%06X" : "%04X", ch);
-      name = buf;
     }
 
-  width = (3 * hbi->pad_x + cols * (hbi->digit_width + hbi->pad_x));
-  lsb = ((double)gi->geometry.width / PANGO_SCALE - width) * .5;
-  lsb = floor (lsb / hbi->pad_x) * hbi->pad_x;
-
   _pango_cairo_renderer_draw_frame (crenderer,
-				    cx + lsb + .5 * hbi->pad_x,
+				    cx + hbi->pad_x * 1.5,
 				    cy + hbi->box_descent - hbi->box_height + hbi->pad_y * 0.5,
-				    width - hbi->pad_x,
+				    (double)gi->geometry.width / PANGO_SCALE - 3 * hbi->pad_x,
 				    (hbi->box_height - hbi->pad_y),
 				    hbi->line_width,
 				    invalid_input);
@@ -500,22 +247,19 @@ _pango_cairo_renderer_draw_unknown_glyph (PangoCairoRenderer *crenderer,
   if (invalid_input)
     goto done;
 
-  x0 = cx + lsb + hbi->pad_x * 2;
-  y0 = cy + hbi->box_descent - hbi->pad_y * 2 - ((hbi->rows - rows) * hbi->digit_height / 2);
+  x0 = cx + hbi->pad_x * 3.0;
+  y0 = cy + hbi->box_descent - hbi->pad_y * 2;
 
-  for (row = 0, p = name; row < rows; row++)
+  for (row = 0; row < rows; row++)
     {
       double y = y0 - (rows - 1 - row) * (hbi->digit_height + hbi->pad_y);
-      for (col = 0; col < cols; col++, p++)
+      for (col = 0; col < cols; col++)
 	{
 	  double x = x0 + col * (hbi->digit_width + hbi->pad_x);
 
-          if (!p)
-            goto done;
-
 	  cairo_move_to (crenderer->cr, x, y);
 
-          hexbox_string[0] = p[0];
+	  hexbox_string[0] = buf[row * cols + col];
 
 	  if (crenderer->do_path)
 	      cairo_text_path (crenderer->cr, hexbox_string);
@@ -598,12 +342,7 @@ pango_cairo_renderer_show_text_glyphs (PangoRenderer        *renderer,
 		      base_y + (double)(gi->geometry.y_offset) / PANGO_SCALE;
 
 	  if (gi->glyph & PANGO_GLYPH_UNKNOWN_FLAG)
-            {
-              if (gi->glyph == (0x20 | PANGO_GLYPH_UNKNOWN_FLAG))
-                ; /* no hex boxes for space, please */
-              else
-	        _pango_cairo_renderer_draw_unknown_glyph (crenderer, font, gi, cx, cy);
-            }
+	    _pango_cairo_renderer_draw_unknown_glyph (crenderer, font, gi, cx, cy);
 	  else
 	    {
 	      cairo_glyphs[count].index = gi->glyph;
@@ -695,7 +434,7 @@ pango_cairo_renderer_draw_glyph_item (PangoRenderer     *renderer,
         num_glyphs = backward ? iter.start_glyph - iter.end_glyph : iter.end_glyph - iter.start_glyph;
 
 	if (num_bytes < 1)
-	  g_warning ("pango_cairo_renderer_draw_glyph_item: bad cluster has num_bytes %d", num_bytes);
+	  g_warning ("pango_cairo_renderer_draw_glyph_item: bad cluster has num_bytess %d", num_bytes);
 	if (num_glyphs < 1)
 	  g_warning ("pango_cairo_renderer_draw_glyph_item: bad cluster has num_glyphs %d", num_glyphs);
 
@@ -959,7 +698,7 @@ pango_cairo_renderer_class_init (PangoCairoRendererClass *klass)
   renderer_class->draw_shape = pango_cairo_renderer_draw_shape;
 }
 
-static PangoCairoRenderer *cached_renderer = NULL; /* MT-safe */
+static PangoCairoRenderer *cached_renderer = NULL;
 G_LOCK_DEFINE_STATIC (cached_renderer);
 
 static PangoCairoRenderer *
@@ -1051,7 +790,6 @@ _pango_cairo_do_glyph_string (cairo_t          *cr,
       pango_renderer_set_color (renderer, PANGO_RENDER_PART_BACKGROUND, NULL);
       pango_renderer_set_color (renderer, PANGO_RENDER_PART_UNDERLINE, NULL);
       pango_renderer_set_color (renderer, PANGO_RENDER_PART_STRIKETHROUGH, NULL);
-      pango_renderer_set_color (renderer, PANGO_RENDER_PART_OVERLINE, NULL);
     }
 
   pango_renderer_draw_glyphs (renderer, font, glyphs, 0, 0);
@@ -1091,7 +829,6 @@ _pango_cairo_do_glyph_item (cairo_t          *cr,
       pango_renderer_set_color (renderer, PANGO_RENDER_PART_BACKGROUND, NULL);
       pango_renderer_set_color (renderer, PANGO_RENDER_PART_UNDERLINE, NULL);
       pango_renderer_set_color (renderer, PANGO_RENDER_PART_STRIKETHROUGH, NULL);
-      pango_renderer_set_color (renderer, PANGO_RENDER_PART_OVERLINE, NULL);
     }
 
   pango_renderer_draw_glyph_item (renderer, text, glyph_item, 0, 0);
