@@ -23,7 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "pangofc-fontmap-private.h"
+#include "pangofc-fontmap.h"
 #include "pangoxft.h"
 #include "pangoxft-private.h"
 
@@ -41,8 +41,6 @@ struct _PangoXftFontMap
 {
   PangoFcFontMap parent_instance;
 
-  guint serial;
-
   Display *display;
   int screen;
 
@@ -61,16 +59,13 @@ struct _PangoXftFontMapClass
   PangoFcFontMapClass parent_class;
 };
 
-static guint         pango_xft_font_map_get_serial         (PangoFontMap         *fontmap);
-static void          pango_xft_font_map_changed            (PangoFontMap         *fontmap);
 static void          pango_xft_font_map_default_substitute (PangoFcFontMap       *fcfontmap,
 							    FcPattern            *pattern);
 static PangoFcFont * pango_xft_font_map_new_font           (PangoFcFontMap       *fcfontmap,
 							    FcPattern            *pattern);
 static void          pango_xft_font_map_finalize           (GObject              *object);
 
-G_LOCK_DEFINE_STATIC (fontmaps);
-static GSList *fontmaps = NULL; /* MT-safe */
+static GSList *fontmaps = NULL;
 
 G_DEFINE_TYPE (PangoXftFontMap, pango_xft_font_map, PANGO_TYPE_FC_FONT_MAP)
 
@@ -78,22 +73,16 @@ static void
 pango_xft_font_map_class_init (PangoXftFontMapClass *class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  PangoFontMapClass *fontmap_class = PANGO_FONT_MAP_CLASS (class);
   PangoFcFontMapClass *fcfontmap_class = PANGO_FC_FONT_MAP_CLASS (class);
 
   gobject_class->finalize  = pango_xft_font_map_finalize;
-
-  fontmap_class->get_serial = pango_xft_font_map_get_serial;
-  fontmap_class->changed = pango_xft_font_map_changed;
-
   fcfontmap_class->default_substitute = pango_xft_font_map_default_substitute;
   fcfontmap_class->new_font = pango_xft_font_map_new_font;
 }
 
 static void
-pango_xft_font_map_init (PangoXftFontMap *xftfontmap)
+pango_xft_font_map_init (PangoXftFontMap *xftfontmap G_GNUC_UNUSED)
 {
-  xftfontmap->serial = 1;
 }
 
 static void
@@ -104,9 +93,7 @@ pango_xft_font_map_finalize (GObject *object)
   if (xftfontmap->renderer)
     g_object_unref (xftfontmap->renderer);
 
-  G_LOCK (fontmaps);
   fontmaps = g_slist_remove (fontmaps, object);
-  G_UNLOCK (fontmaps);
 
   if (xftfontmap->substitute_destroy)
     xftfontmap->substitute_destroy (xftfontmap->substitute_data);
@@ -115,31 +102,12 @@ pango_xft_font_map_finalize (GObject *object)
 }
 
 
-static guint
-pango_xft_font_map_get_serial (PangoFontMap *fontmap)
-{
-  PangoXftFontMap *xftfontmap = PANGO_XFT_FONT_MAP (fontmap);
-
-  return xftfontmap->serial;
-}
-
-static void
-pango_xft_font_map_changed (PangoFontMap *fontmap)
-{
-  PangoXftFontMap *xftfontmap = PANGO_XFT_FONT_MAP (fontmap);
-
-  xftfontmap->serial++;
-  if (xftfontmap->serial == 0)
-    xftfontmap->serial++;
-}
-
 static PangoFontMap *
 pango_xft_find_font_map (Display *display,
 			 int      screen)
 {
   GSList *tmp_list;
 
-  G_LOCK (fontmaps);
   tmp_list = fontmaps;
   while (tmp_list)
     {
@@ -147,14 +115,10 @@ pango_xft_find_font_map (Display *display,
 
       if (xftfontmap->display == display &&
 	  xftfontmap->screen == screen)
-        {
-          G_UNLOCK (fontmaps);
-	  return PANGO_FONT_MAP (xftfontmap);
-        }
+	return PANGO_FONT_MAP (xftfontmap);
 
       tmp_list = tmp_list->next;
     }
-  G_UNLOCK (fontmaps);
 
   return NULL;
 }
@@ -162,7 +126,7 @@ pango_xft_find_font_map (Display *display,
 /*
  * Hackery to set up notification when a Display is closed
  */
-static GSList *registered_displays; /* MT-safe, protected by fontmaps lock */
+static GSList *registered_displays;
 
 static int
 close_display_cb (Display   *display,
@@ -170,10 +134,7 @@ close_display_cb (Display   *display,
 {
   GSList *tmp_list;
 
-  G_LOCK (fontmaps);
-  tmp_list = g_slist_copy (fontmaps);
-  G_UNLOCK (fontmaps);
-
+  tmp_list = fontmaps;
   while (tmp_list)
     {
       PangoXftFontMap *xftfontmap = tmp_list->data;
@@ -183,8 +144,6 @@ close_display_cb (Display   *display,
 	pango_xft_shutdown_display (display, xftfontmap->screen);
     }
 
-  g_slist_free (tmp_list);
-
   registered_displays = g_slist_remove (registered_displays, display);
 
   return 0;
@@ -213,7 +172,7 @@ register_display (Display *display)
  * @display: an X display
  * @screen: the screen number of a screen within @display
  *
- * Returns the #PangoXftFontMap for the given display and screen.
+ * Returns the #PangoXftFontmap for the given display and screen.
  * The fontmap is owned by Pango and will be valid until
  * the display is closed.
  *
@@ -234,19 +193,18 @@ pango_xft_get_font_map (Display *display,
   if (fontmap)
     return fontmap;
 
+  /* Make sure that the type system is initialized */
+  g_type_init ();
+
   xftfontmap = (PangoXftFontMap *)g_object_new (PANGO_TYPE_XFT_FONT_MAP, NULL);
 
   xftfontmap->display = display;
   xftfontmap->screen = screen;
 
-  G_LOCK (fontmaps);
-
   register_display (display);
 
   fontmaps = g_slist_prepend (fontmaps, xftfontmap);
 
-  G_UNLOCK (fontmaps);
-
   return PANGO_FONT_MAP (xftfontmap);
 }
 
@@ -273,9 +231,7 @@ pango_xft_shutdown_display (Display *display,
     {
       PangoXftFontMap *xftfontmap = PANGO_XFT_FONT_MAP (fontmap);
 
-      G_LOCK (fontmaps);
       fontmaps = g_slist_remove (fontmaps, fontmap);
-      G_UNLOCK (fontmaps);
       pango_fc_font_map_shutdown (PANGO_FC_FONT_MAP (fontmap));
 
       xftfontmap->display = NULL;
@@ -308,10 +264,6 @@ pango_xft_set_default_substitute (Display                *display,
 {
   PangoXftFontMap *xftfontmap = (PangoXftFontMap *)pango_xft_get_font_map (display, screen);
 
-  xftfontmap->serial++;
-  if (xftfontmap->serial == 0)
-    xftfontmap->serial++;
-
   if (xftfontmap->substitute_destroy)
     xftfontmap->substitute_destroy (xftfontmap->substitute_data);
 
@@ -341,9 +293,6 @@ pango_xft_substitute_changed (Display *display,
 {
   PangoXftFontMap *xftfontmap = (PangoXftFontMap *)pango_xft_get_font_map (display, screen);
 
-  xftfontmap->serial++;
-  if (xftfontmap->serial == 0)
-    xftfontmap->serial++;
   pango_fc_font_map_cache_clear (PANGO_FC_FONT_MAP (xftfontmap));
 }
 
@@ -384,7 +333,7 @@ pango_xft_get_context (Display *display,
 
 /**
  * _pango_xft_font_map_get_renderer:
- * @fontmap: a #PangoXftFontMap
+ * @fontmap: a #PangoXftFontmap
  *
  * Gets the singleton #PangoXFTRenderer for this fontmap.
  *
